<?php

    namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander;
    
    use Doctrine\ORM\EntityManager;
    use HCM\Util\HCMParameters;
    use HCM\Util\FPA;
    use HCM\FPA\Entity\FpatipoassoceveEntity;
    use HCM\FPA\Registro\FPARegistro;
    use HCM\Util\DateUtil;
    use HCM\Util\Repositories;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\FpaHoleritemagnet;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\RegistroHolerite;
    use Teknisa\Model\Gestaopessoas\Entities\FpaFormholerite; 
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro0HoleriteSantander;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro1HoleriteSantander;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro2HoleriteSantander;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro3HoleriteSantander;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro9HoleriteSantander;
    

class HoleriteSantander{
    
    const CODIGO_BANCO_SANTANDER = '033';

    private $dadosRemunVinculo;
    private $dadosRemunEstrutura;
        
    public function __construct(
        FpaFormholerite $formulario, HcmParameters $hcmParameters, EntityManager $entityManager,$horeliteMagnetico
    ){
        error_reporting(E_ERROR | E_PARSE);
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;

        $this->nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->mesCompetencia = $this->hcmParameters->getDtCompetencia();

        $this->dtInicio = $this->mesCompetencia;

        $this->dtFim = DateUtil::adicionaIntervalo($this->mesCompetencia,1,DateUtil::MES);
        $this->dtFim = DateUtil::subtraiIntervalo($this->dtFim,1);

        $this->holeriteMagnetico = $horeliteMagnetico;

        $this->registroHolerite = new RegistroHolerite();
        $this->registro0 = new Registro0HoleriteSantander();
        $this->registro1 = new Registro1HoleriteSantander();
        $this->registro2 = new Registro2HoleriteSantander();
        $this->registro3 = new Registro3HoleriteSantander();
        $this->registro9 = new Registro9HoleriteSantander();

        $this->inicializaVariaveis($formulario);
    }


    private function inicializaVariaveis($formulario){

        $this->nrTpMovimentoFormulario = $formulario->getNrtipomoviment();
        $this->holeriteTeste = $formulario->getIdteste();
        $this->codConvenio = $formulario->getCodconvenio();
        $this->dtPagamento = $formulario->getDtpagamento();
        $this->idtparquivo = $formulario->getIdtparquivo();
        $this->nrlote = $formulario->getNrlote();
        $this->idTpPagamento = $formulario->getIdtppagamento();
        
        switch ($this->idtparquivo) {
            case 'T':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_T;
                break;
            case 'C':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_C;
                break;
            case 'S':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_S;
                break;
            case 'A':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_A;
                break;
            case 'E':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_E;
                break;
            case 'D':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_D;
                break;
            case 'X':
                $this->listaRegistros = RegistroHolerite::LISTA_REGISTROS_TIPO_ARQ_X;
                break;
        }
    }

    
    public function geraHolerite( $estrutura,  $pessoa,  $vinculo, $totalizar = false){
        $this->estrutura = $estrutura;
        $this->pessoa = $pessoa;
        $this->vinculo = $vinculo;
        $this->dadosRemunVinculo = array();
        // $this->dadosRemunEstrutura = array();

        $ultimoSequencial = $this->holeriteMagnetico->retornaUltimoSequencialHolerite(
            $this->mesCompetencia, $this->nrorg, self::CODIGO_BANCO_SANTANDER, $estrutura->getNrestruturam()
        );
        if (!empty($ultimoSequencial)) {
            $this->nrSeqArquivo = $ultimoSequencial;
            $this->nrSeqArquivo++;
        } else {
            $this->nrSeqArquivo = FPARegistro::CARACTER_UM;
        }
        
        /* Verifica na lista de registros, quais registros serão gerados */
        if (strpos($this->listaRegistros, Registro0HoleriteSantander::REGISTRO_0) !== false && !$totalizar) {
            $this->preparaRegistro0();
        }
        if (strpos($this->listaRegistros, Registro1HoleriteSantander::REGISTRO_1) !== false && !$totalizar) {
            $this->preparaRegistro1();
        }
        if (strpos($this->listaRegistros, Registro2HoleriteSantander::REGISTRO_2) !== false && !$totalizar) {
            $this->preparaRegistro2();
        }
        if (strpos($this->listaRegistros, Registro3HoleriteSantander::REGISTRO_3) !== false && !$totalizar) {
            $this->preparaRegistro3();
        }
        if (strpos($this->listaRegistros, Registro9HoleriteSantander::REGISTRO_9) !== false && $totalizar) {
            $this->preparaRegistro9();
        }
    }

    private function preparaRegistro0() {
        $posicao = $this->holeriteMagnetico->procuraRegistroLista(
            $this->estrutura->nrInscricao, Registro0HoleriteSantander::REGISTRO_0, self::CODIGO_BANCO_SANTANDER,
            $this->nrTpMovimentoFormulario
        );

        if (!$posicao) {

            $this->registro0->setEstrutura($this->estrutura);
            $this->registro0->setNrseqarquivo($this->nrSeqArquivo);
            $this->registro0->setCodConvenioBanco($this->codConvenio);
            $this->registro0->setDtPagamento($this->dtPagamento);
            $this->registro0->setCodBanco(self::CODIGO_BANCO_SANTANDER);
            $this->registro0->setTpArquivo($this->idtparquivo);
            if (empty($this->nrlote)) {
                $this->registro0->setNrLote($this->nrSeqArquivo);
            } else {
                $this->registro0->setNrLote($this->nrlote);
            }
            $this->registro0->setIdtppagamento($this->idTpPagamento);

            $this->registro0->geraRegistro0();
            $this->holeriteMagnetico->insereRegistroLista(
                $this->nrorg, $this->mesCompetencia, self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario,
                $this->idtparquivo, Registro0HoleriteSantander::REGISTRO_0,
                $this->estrutura->nrInscricao, $this->registro0->getLinhaRegistro(),
                $this->nrSeqArquivo, $this->nrSeqArquivo, 0
            );

            $this->registro0->setLinhaRegistro();
        }
    }


    private function preparaRegistro1() {

        $posicao = $this->holeriteMagnetico->procuraRegistroLista(
            $this->estrutura->nrInscricao, Registro1HoleriteSantander::REGISTRO_1, self::CODIGO_BANCO_SANTANDER,
            $this->nrTpMovimentoFormulario
        );
        if (!$posicao) {

            $this->registro1->setEstrutura($this->estrutura);
            $this->registro1->setCodConvenioBanco($this->codConvenio);
            $this->registro1->setPessoa($this->pessoa);
            $this->registro1->setVinculo($this->vinculo);

            $this->registro1->geraRegistro1();
            
            $this->holeriteMagnetico->persisteRegistro(
                $this->nrorg, DateUtil::getDataDeString($this->mesCompetencia,DateUtil::FORMATO_BRASILEIRO,true), self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario,
                $this->idtparquivo, Registro1HoleriteSantander::REGISTRO_1,
                $this->estrutura->nrInscricao, $this->nrSeqArquivo, $this->nrSeqArquivo, $this->registro1->getLinhaRegistro(),
                $this->vinculo->getNrvinculom(), $this->pessoa->historico->getNrcpfpessoa(), 0
            );
            
            $this->registro1->setLinhaRegistro();
        }
    }


    private function preparaRegistro2() {
        $sequencial = 1;
        $this->registro2->setPessoa($this->pessoa);
        $this->registro2->setVinculo($this->vinculo);
        foreach ($this->vinculo->itensCalculo as $key=>$itemCalculo) {
            $itemCalculo->evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->findOneBy(array("nrorg"=> [$this->hcmParameters->getNrorgPadrao(),$this->hcmParameters->getNrorgTrabalho()],"nreventom"=>$itemCalculo->getNreventom()));
            if ( ( $itemCalculo->evento->getIdtipoevento() == 'PROVENTO'
                   || $itemCalculo->evento->getIdtipoevento() == 'DESCONTO'
                   || $itemCalculo->evento->getIdtipoevento() == 'TOTAL'
                   || $itemCalculo->evento->getNreventom() == RegistroHolerite::REF_FGTS_MES ) &&
                    $itemCalculo->getIdevendemoncalc() == 'N' ) {

                if($itemCalculo->evento->getIdtipoevento() != 'TOTAL' && $itemCalculo->evento->getNreventom() != RegistroHolerite::REF_FGTS_MES) {
    
                    $this->registro2->setItemCalculo($itemCalculo);
                    $this->registro2->setSequencial($sequencial);
    
                    
                    $this->registro2->geraRegistro2();
                    $this->holeriteMagnetico->insereRegistroLista(
                        $this->nrorg, $this->mesCompetencia, self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario,
                        $this->idtparquivo, Registro2HoleriteSantander::REGISTRO_2,
                        $this->estrutura->nrInscricao, $this->registro2->getLinhaRegistro(),
                        $this->nrSeqArquivo, $key, $this->vinculo->getNrvinculom(), $this->pessoa->historico->getNrcpfpessoa()
                    );
    
                    
                    $this->registro2->setLinhaRegistro();
    
                    $this->consolidaDadosVinculo(RegistroHolerite::REF_QUANT_LANC, 1);
                    $sequencial++;
                }
                
                $this->consolidaItemVinculo($itemCalculo->getNreventom(), $itemCalculo->getVreventofolha(), $itemCalculo->evento->getIdtipoevento());
            }
        }
    }


    private function preparaRegistro3() {

        $this->registro3->setPessoa($this->pessoa);
        $this->registro3->setVinculo($this->vinculo);
        $this->registro3->setDadosVinculo($this->dadosRemunVinculo);

        
        $this->registro3->geraRegistro3();
        $this->holeriteMagnetico->persisteRegistro(
            $this->nrorg, DateUtil::getDataDeString($this->mesCompetencia,DateUtil::FORMATO_BRASILEIRO,true), self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario,
            $this->idtparquivo, Registro3HoleriteSantander::REGISTRO_3,
            $this->estrutura->nrInscricao, $this->nrSeqArquivo, $this->nrSeqArquivo, $this->registro3->getLinhaRegistro(),
            $this->vinculo->getNrvinculom(), $this->pessoa->historico->getNrcpfpessoa(), 0
        );
        
        $this->registro3->setLinhaRegistro();

        $this->consolidaItemEstrutura();
    }


    private function preparaRegistro9() {
        $posicao = $this->holeriteMagnetico->procuraRegistroLista(
            $this->estrutura->nrInscricao, Registro9HoleriteSantander::REGISTRO_9, self::CODIGO_BANCO_SANTANDER
            , $this->nrTpMovimentoFormulario
        );
        
        if ($posicao === false) {

            $this->registro9->setEstrutura($this->estrutura);
            $this->registro9->setCodConvenioBanco($this->codConvenio);
            $this->registro9->setDadosEstrutura($this->dadosRemunEstrutura);

            $this->registro9->geraRegistro9();

            $this->holeriteMagnetico->insereRegistroLista(
                $this->nrorg, $this->mesCompetencia, self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario,
                $this->idtparquivo, Registro9HoleriteSantander::REGISTRO_9,
                $this->estrutura->nrInscricao, $this->registro9->getLinhaRegistro(),
                $this->nrSeqArquivo
            );

            $this->registro9->setLinhaRegistro();
        }
    }


    public function modificaLinhasConsolidacaoSantander($tipoRegistro, $linha){
        if ($tipoRegistro === Registro9HoleriteSantander::REGISTRO_9) {

            
            $qtdRegistrosArquivo = $this->holeriteMagnetico->totalRegistrosPorTipos(
                DateUtil::getDataDeString($this->mesCompetencia,DateUtil::FORMATO_BRASILEIRO,true), $this->nrorg, RegistroHolerite::LISTA_TIPO_REGISTROS_ARQUIVO,
                self::CODIGO_BANCO_SANTANDER, $this->nrTpMovimentoFormulario
            );
            
            $linha = $this->registro9->alteraDado(
                $linha, (string)$qtdRegistrosArquivo, (81-1), 10, Registro9HoleriteSantander::CARACTER_ZERO, Registro9HoleriteSantander::PREENCHE_ESQUERDA
            );
        }

        return $linha;
    }

    private function consolidaItemVinculo($nrevento, $vrevento, $idtipoevento)
    {
        switch ($nrevento) {
            case RegistroHolerite::REF_BASE_IRRF :
            case FpatipoassoceveEntity::ASSOC_INCID_IRRF_ADIANT:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_BASE_IRRF, $vrevento);
                break;
            case RegistroHolerite::REF_BASE_INSS :
                case FpatipoassoceveEntity::ASSOC_INCID_INSS_ADIANT:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_BASE_INSS, $vrevento);
                break;
            case RegistroHolerite::REF_BASE_FGTS :
            case FpatipoassoceveEntity::ASSOC_INCID_FGTS_ADIANT:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_BASE_FGTS, $vrevento);
                break;
            case RegistroHolerite::REF_FGTS_MES :
            case FpatipoassoceveEntity::ASSOC_FGTS_ADIANTAMENTO:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_FGTS_MES, $vrevento);
                break;
        }

        $vrevento = round($vrevento, 2);
        switch ($idtipoevento) {
            case RegistroHolerite::TIPO_EVENTO_PROVENTO:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_TOTAL_CREDITO, $vrevento);
                $this->consolidaDadosVinculo(RegistroHolerite::REF_TOTAL_LIQUIDO, $vrevento); 
                break;
            case RegistroHolerite::TIPO_EVENTO_DESCONTO:
                $this->consolidaDadosVinculo(RegistroHolerite::REF_TOTAL_DEBITO, $vrevento);
                $this->consolidaDadosVinculo(RegistroHolerite::REF_TOTAL_LIQUIDO, -1 * $vrevento); 
                break;
        }
    }

    private function consolidaItemEstrutura()
    {
        $this->consolidaDadosEstrut(RegistroHolerite::REF_TOTAL_CREDITO, $this->dadosRemunVinculo[RegistroHolerite::REF_TOTAL_CREDITO]);
        $this->consolidaDadosEstrut(RegistroHolerite::REF_TOTAL_DEBITO, $this->dadosRemunVinculo[RegistroHolerite::REF_TOTAL_DEBITO]);
        $this->consolidaDadosEstrut(RegistroHolerite::REF_TOTAL_LIQUIDO, $this->dadosRemunVinculo[RegistroHolerite::REF_TOTAL_LIQUIDO]);

        $this->consolidaDadosEstrut(RegistroHolerite::REF_QUANT_LANC, $this->dadosRemunVinculo[RegistroHolerite::REF_QUANT_LANC]);
    }

    private function consolidaDadosEstrut($referencia, $valor)
    {
        //if(array_key_exists($referencia, $this->dadosRemunEstrutura)) {
            $this->dadosRemunEstrutura[$referencia] += $valor;
        /*} else {
            $this->dadosRemunEstrutura[$referencia] = $valor;
        }*/
    }

    public function getDadosEstrut($referencia = null)
    {
        if ($referencia == null) {
            return $this->dadosRemunEstrutura;
        }
        else {
            return $this->dadosRemunEstrutura[$referencia];
        }
    }

    private function consolidaDadosVinculo($referencia, $valor)
    {   
        if(array_key_exists($referencia,$this->dadosRemunVinculo))
            $this->dadosRemunVinculo[$referencia] += $valor;
        else {
            $this->dadosRemunVinculo[$referencia] = $valor;
        }
    }

    public function getDadosVinculo($referencia = null)
    {
        if ($referencia == null) {
            return $this->dadosRemunVinculo;
        }
        else {
            return $this->dadosRemunVinculo[$referencia];
        }
    }

}