<?php

namespace HCM\Geral\Controllers;

use HCM\Util\Messages;
use HCM\Geral\Services\ConsultaQualificaService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ConsultaQualificaController extends Crud{

    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ConsultaQualificaService $consultaQualificaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->consultaQualificaService = $consultaQualificaService;
        parent::__construct($dataSourceManager);
    }

    public function importarCQC(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $registros = array();
                foreach ($row['DSPATH'] as $arquivo) {
                    $row['DSPATH'] = [$arquivo];
                    $registros = array_merge($registros, $this->consultaQualificaService->importarCQC($row));
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getEsocialCQC', $registros));
                $response->addNotification(new DTO\Response\Notification(Messages::HCM00001_GER_IMPORTACAO_SUCESS));
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 