<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Services\PessoaService;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\ParcnegocioService;
use HCM\Util\Util;
use HCM\Util\Messages;

class ParceiroNegocioController extends Crud {

    protected $dataSourceName = 'parcnegocio';
    protected $dataSourceNameEndereco = 'enderecoparc_get';
    protected $dataSourceNamePessoa = 'getParcNegocioPessoa';
    protected $dataSourceName2 = 'parcnegocioRelacioando';
    protected $dataSourceName3 = 'parcnegocio_get';
    protected $dataSourceName4 = 'getParcNegocioRespESocial';
    protected $nmArquivoPlanilhaFC = 'FormaDeComunicacao';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDTIPOPARCPRINCIPAL', 'P_NRORGPADRAO', 'P_NRPARCNEGOCIO', 'EXISTE_VINCULOS', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, ParcnegocioService $parcnegocioService, PessoaService $pessoaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->parcnegocioService = $parcnegocioService;
        $this->pessoaService = $pessoaService;
        parent::__construct($dataSourceManager);
    }
    
    public function importarPlanilhaContaCorrente(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->parcnegocioService->importarPlanilhaContaCorrente($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getParcNegocio (DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(0);
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $newFilterData->setPageSize(5000000);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getParcNegocioPessoa (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNamePessoa);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getParcNegocioTodos (DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(0);
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $newFilterData->setPageSize(5000000);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row) {
            if(isset($row['CDTIPOINSCRICAO']) && !empty($row['CDTIPOINSCRICAO'])){
                if( $row['CDTIPOINSCRICAO'] == 'CPF' && isset($row['NRINSCRICAOPARC']) && !empty($row['NRINSCRICAOPARC']) ) {
                    $cpf = preg_replace("/[^0-9]/", "", $row['NRINSCRICAOPARC']);
                    $cpf = str_pad($cpf, 11, "0", STR_PAD_LEFT);
                    $row['NRINSCRICAOPARC'] = Util::formatCnpjCpf($cpf);
                }
                elseif( $row['CDTIPOINSCRICAO'] == 'CNPJ' && isset($row['NRINSCRICAOPARC']) && !empty($row['NRINSCRICAOPARC']) ){
                    $cnpj = preg_replace("/[^0-9]/", "", $row['NRINSCRICAOPARC']);
                    $cnpj = str_pad($cnpj, 14, "0", STR_PAD_LEFT);
                    $row['NRINSCRICAOPARC'] = Util::formatCnpjCpf($cnpj);
                }    
            }
            
        }       
        
        $response->addDataSet(new DataSet($this->dataSourceName3, $rows));    
    }
    
    public function getParcNegocioTodosSemMascara (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        
        $rows = $dataSet->getRows();
        
        $response->addDataSet(new DataSet($this->dataSourceName3, $rows));    
    }
    
    public function getParcNegocioRespESocial (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName4);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getParcNegocioRelacionado (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getTipoEndereco(DTO\Request\Filter $request, DTO\Response $response){
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('tipoendereco');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function setParcNegocio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'parcnegocio_set';
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPARCNEGOCIO', $row)) {
                    if ($row['NRPARCNEGOCIO'] === '') {
                        $row['NRPARCNEGOCIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PARCNEGOCIO', 1, 12, true); 
                    }
                     if(!array_key_exists('NRPARCMONBIO', $row)){
                    $row['NRPARCMONBIO'] = null;
                    }
                      if(!array_key_exists('NRPARCREGAMB', $row)){
                    $row['NRPARCREGAMB'] = null;
                    }
                    } else {
                      $row['NRPARCNEGOCIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PARCNEGOCIO', 1, 12, true); 
                    }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getEnderecoParc(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        foreach($conditions as &$condition){
            if($condition['columnName'] == 'NRCEPENDERECO') {
                $condition['value'] =  str_replace(array('.','-', '%'),'', $condition['value']);
            }
        }
        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );
        $filterData->setDataSourceName($this->dataSourceNameEndereco);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setEnderecoParc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "enderecoparc";
            $jaExistente = null; 
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRENDERECOPARC', $row)) {
                    if ($row['NRENDERECOPARC'] === '') {
                        $row['NRENDERECOPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ENDERECOPARC', 1, 12, true); 
                    }
                } else {
                    $row['NRENDERECOPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ENDERECOPARC', 1, 12, true); 
                }
                if(!array_key_exists('CDPAIS', $row)){
                    $row['CDPAIS'] = null;
                }
                if(!array_key_exists('SGESTADO', $row)){
                    $row['SGESTADO'] = null;
                }
                if(!array_key_exists('CDMUNICIPIO', $row)){
                    $row['CDMUNICIPIO'] = null;
                }
                if(!array_key_exists('CDLOGRADOURO', $row)){
                    $row['CDLOGRADOURO'] = null;
                }
                if(!array_key_exists('DSLONGITUDE', $row)){
                    $row['DSLONGITUDE'] = null;
                }
                if(!array_key_exists('DSLATITUDE', $row)){
                    $row['DSLATITUDE'] = null;
                }
                if(!array_key_exists('CDBAIRRO', $row)){
                    $row['CDBAIRRO'] = null;
                }
                if(array_key_exists('NMBAIRROENDERECO', $row)){
                    $row['NMBAIRROENDERECO'] = Util::removeCaracteresEspeciais($row['NMBAIRROENDERECO']);
                }
                if(!array_key_exists('DSCOMPLEENDERECO', $row)){
                    $row['DSCOMPLEENDERECO'] = null;
                }
                if(!array_key_exists('NRCEPENDERECO', $row)){
                    $row['NRCEPENDERECO'] = null;
                } else {
                    $row['NRCEPENDERECO'] = str_replace(['-', '.'], "", $row['NRCEPENDERECO']);
                }
                if(!array_key_exists('DSREFERENCIAENDE', $row)){
                    $row['DSREFERENCIAENDE'] = null;
                }
                if(!array_key_exists('DSENDERECO', $row)){
                    $row['DSENDERECO'] = null;
                } else {
                    $row['DSENDERECO'] = Util::removeCaracteresEspeciais($row['DSENDERECO']);
                }
                if(!array_key_exists('NRIMOVELENDERECO', $row)){
                    $row['NRIMOVELENDERECO'] = null;
                }
                if(!array_key_exists('CDPAIS', $row)){
                    $row['CDPAIS'] = null;
                }
                if(!array_key_exists('SGESTADO', $row)){
                    $row['SGESTADO'] = null;
                }
                
                if(!$row['CDLOGRADOURO'] && $row['DSLOGRADOURO']){
                    $logradouro = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findBy(array('nrorg' => array($row['NRORG'], '0')));
                    foreach ($logradouro as $key => $value) {
                        if(strtoupper($value->getDslogradouro()) == strtoupper($row['DSLOGRADOURO'])){
                            $row['CDLOGRADOURO'] = $value->getCdlogradouro();
                            break;
                        }
                    }
                }
                
                if($row["__is_new"]){ 
                    $jaExistente = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findBy(
                            array("nrparcnegocio" => $row['NRPARCNEGOCIO'], "cdtipoendereco" => $row['CDTIPOENDERECO'], "nrorg"=>$row['NRORG'])
                    );
                }else{
                    $rowAntesDeEditar= $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findBy(array("nrenderecoparc" => $row['NRENDERECOPARC'], "nrorg"=>$row['NRORG']));
                    if($rowAntesDeEditar[0]->getCdtipoendereco() != $row['CDTIPOENDERECO']){
                        $jaExistente = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findBy(
                                array("nrparcnegocio" => $row['NRPARCNEGOCIO'], "cdtipoendereco" => $row['CDTIPOENDERECO'], "nrorg"=>$row['NRORG'])
                        );
                    }    
                }
                
            }
            if( (is_array($jaExistente) && count($jaExistente)>0) || (!is_array($jaExistente) && isset($jaExistente)) ){
                $response->addNotification(new DTO\Response\Notification(Messages::HCM00048_GPE_PESSOA_ENDERECO_EXISTENTE_ALERT, DTO\Response\Notification::TYPE_ALERT, 'registroExistente'));
            }else{
                if(!$row['CDPAIS'] == '' && !$row['CDPAIS'] == null && !$row['SGESTADO'] == '' && !$row['SGESTADO'] == null && !$row['CDMUNICIPIO'] == '' && !$row['CDMUNICIPIO'] == null){
                    $this->pessoaService->validaMunicipio($row);
                }
                
                parent::save($request, $response);
                $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrorg' => $row['NRORG'], 'nrparcnegocio' => $row['NRPARCNEGOCIO']));
                if($pessoa){
                    $this->pessoaService->verificaHistoricoCompetenciaPessoa($row['NRORG'], $pessoa->getNrpessoa());
                    $this->pessoaService->atualizaHistoricoPessoa($row['NRORG'], $pessoa->getNrpessoa());
                }
            }    
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEnderecoParc(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "enderecoparc";
        parent::delete($request, $response);
    }
    
    public function getFormaComunicacaoParc(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "comunicaparc_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setFormaComunicacaoParc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "comunicaparc";
            $jaExistente = null;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCOMUNICAPARC', $row)) {
                    if ($row['NRCOMUNICAPARC'] === '') {
                        $row['NRCOMUNICAPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'COMUNICAPARC', 1, 12, true); 
                    }
                } else {
                    $row['NRCOMUNICAPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'COMUNICAPARC', 1, 12, true); 
                }
                if(!array_key_exists('NRENDERECOPARC', $row)){
                    $row['NRENDERECOPARC'] = null;
                }
                if(!array_key_exists('CDCOMPLECOMUPARC', $row)){
                    $row['CDCOMPLECOMUPARC'] = null;
                }
                
                if($row["__is_new"]){ 
                    $jaExistente = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(
                            array("nrparcnegocio" => $row['NRPARCNEGOCIO'], "cdformacomu" => $row['CDFORMACOMU'], "nrorg"=>$row['NRORG'])
                    );
                }else{
                    $rowAntesDeEditar= $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(array("nrcomunicaparc" => $row['NRCOMUNICAPARC'], "nrorg"=>$row['NRORG']));
                    if((int)$rowAntesDeEditar[0]->getCdformacomu() != (int)$row['CDFORMACOMU']){
                        $jaExistente = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(
                                array("nrparcnegocio" => $row['NRPARCNEGOCIO'], "cdformacomu" => $row['CDFORMACOMU'], "nrorg"=>$row['NRORG'])
                        );
                    }    
                }
            }
            
            if( (is_array($jaExistente) && count($jaExistente)>0) || (!is_array($jaExistente) && isset($jaExistente)) ){
                $response->addNotification(new DTO\Response\Notification(Messages::HCM00048_GPE_PESSOA_FORMACOMUNICACAO_EXISTENTE_ALERT, DTO\Response\Notification::TYPE_ALERT, 'registroExistente'));
            }else{                
                parent::save($request, $response);
                $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrorg' => $row['NRORG'], 'nrparcnegocio' => $row['NRPARCNEGOCIO']));
                if($pessoa){    
                    $this->pessoaService->verificaHistoricoCompetenciaPessoa($row['NRORG'], $pessoa->getNrpessoa());
                    $this->pessoaService->atualizaHistoricoPessoa($row['NRORG'], $pessoa->getNrpessoa());
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteFormaComunicacaoParc(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "comunicaparc";
        parent::delete($request, $response);
    }
    
    public function cadAutoFormaComu(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->parcnegocioService->cadAutoFormaComu($row);
            }   
            $response->addNotification(new DTO\Response\Notification('cadAutoFormaComu'), DTO\Response\Notification::TYPE_SUCCESS);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getContaCorrenteParc(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "contcorrparc_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setContaCorrenteParc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "contcorrparc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCONTCORRPARC', $row)) {
                    if ($row['NRCONTCORRPARC'] === '') {
                        $row['NRCONTCORRPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'CONTCORRPARC', 1, 12, true); 
                    }
                } else {
                    $row['NRCONTCORRPARC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'CONTCORRPARC', 1, 12, true); 
                }
                if(!array_key_exists('CDREDUCONT', $row)){
                    $row['CDREDUCONT'] = null;
                }
                if(array_key_exists('CDCONTCORR', $row)){
                    $row['CDCONTCORR'] = str_replace(['-', '.'], "", $row['CDCONTCORR']);
                }
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteContaCorrenteParc(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "contcorrparc";
        parent::delete($request, $response);
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function getParcnegocioVinculos($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_CDTIPOPARCPRINCIPAL', 'P_ITEMSPERPAGE', 'P_NRORG'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->parcnegocioService->getParcnegocioVinculos($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getParcnegocioVinculos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
    
    /*Planilhas*/
    /*Forma de comunicação*/
    public function baixarPlanilhaFormaComunicacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $file = $this->parcnegocioService->baixarPlanilhaFormaComunicacao(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->nmArquivoPlanilhaFC);
                
    
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaFormaComunicacao(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->parcnegocioService->importarPlanilhaFormaComunicacao($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
