<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\SolicitacaoService;
use HCM\Geral\Util\ExpectedException;
use HCM\Util\{ HCMParameters, Util };
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class SolicitacaoController extends Crud {

    const IDAPROVACAO_APROVADO = 'A';
    const IDAPROVACAO_REPROVADO = 'R';

    const PERFACESTIPOSOLI_ARRAY_FIELDS = ['NRESTRUTLEGAL', 'NRESTRUTGEREN', 'NRESTRUTSIND', 'NROCUPACAOM', 'NRTIPOVINCULOM'];

    /** @var SolicitacaoService */
    protected $solicitacaoService;

    protected $dataSourceName = 'gpe_solicitacao';
    protected $dataSourceNameGet = 'gpe_solicitacao_get';
    protected $dataSourceNameAprov = 'gpe_aprovasolicitacao_get';
    protected $dataSourceNameTipo = 'gpe_tiposolicitacao';
    protected $dataSourceNameItem = 'gpe_itemsolicitacao';
    protected $dataSourceNamePerf = 'gpe_perfacestiposoli';
    protected $dataSourceNamePerf_get = 'gpe_perfacestiposoli_get';
    protected $dataSourceNameFaseAtual = 'getFaseAtual';
    protected $dataSourceNameFaseTram = 'getFasetramsoli';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_CDOPERADOR', 'P_DTMESCOMPETENC', 'P_NRVINCULOM', 'P_NRSOLICITACAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, SolicitacaoService $solicitacaoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->solicitacaoService = $solicitacaoService;
        parent::__construct($dataSourceManager);
    }

    public function getTiposolicitacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameTipo);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getFaseAtual(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameFaseAtual);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getSolicitacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getAprovaSolicitacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameAprov);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getItemsolicitacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameItem);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getPerfacestiposoli (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNamePerf_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $dataSet = Util::dataSetRowsStringItemsToArray($dataSet, self::PERFACESTIPOSOLI_ARRAY_FIELDS);
        $response->addDataSet($dataSet);
    }

    public function getFasetramsoli (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameFaseTram);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function validaItemSolicitacao(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach ($request->getDataSet()->getRows() as &$row) {
            switch($row['NRTIPOSOLICITACAO']){
                case 5:
                    $result = $this->solicitacaoService->validaDemissao($row);
                    $message = "";
                    if(!empty($result)){
                        foreach ($result as $res) {
                            $message .= $res."<br><br>";
                        }
                        $message .= "Deseja continuar?";
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING));
                    }
                    break;
                case 8:
                    $result = $this->solicitacaoService->validaAlteraEndereco($row);
                    if(!$result)
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Já existe um endereco deste tipo para o vínculo, deseja atualizá-lo?", DTO\Response\Message::TYPE_WARNING));
                    break;
                case 16:
                    $result = $this->solicitacaoService->validaAlteraComunicacao($row);
                    if(!$result)
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Já existe uma forma de comunicação deste tipo para o vínculo, deseja atualizá-la?", DTO\Response\Message::TYPE_WARNING));
                    break;
            }
        }
    }

    public function setSolicitacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                /* Insere uma nova solicitação */
                if ($row['__is_new'] === true) {
                    if (!array_key_exists('DSOBSSOLICITACAO', $row)) {
                        $row['DSOBSSOLICITACAO'] = '';
                    }
                    $nrorg = $this->hcmParameters->getNrorgTrabalho();
                    $row['NRSOLICITACAO'] = \Geral\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 22, false);
                    
                    //Se for uma Solicitação de Alteração Salarial junto com uma Solicitação de Alteração de Ocupação
                    if($row['NRTIPOSOLICITACAO'] == SolicitacaoService::TP_SOLI_SALARIO && isset($row['NROCUPACAOM_SAL'])  &&  $row['NROCUPACAOM_SAL'] != ""){
                        if(!$this->solicitacaoService->validaPerfildeAcesso(SolicitacaoService::TP_SOLI_OCUPACAO)) {
                            $erro["Bloquear"] = true;
                            $erro["Texto"] = "Não existe perfil de acesso para solicitar uma Alteracao de Ocupacao";
                        } else if (!$this->solicitacaoService->validaPerfildeAcesso(SolicitacaoService::TP_SOLI_SALARIO)) {
                            $erro["Bloquear"] = true;
                            $erro["Texto"] = "Não existe perfil de acesso para solicitar uma Alteracao Salarial";
                        } else {
                            //Insere a Solicitação de Alteração Salarial
                            $erro = $this->solicitacaoService->executaInsertSolicitacao($row);
                            //Insere a Solicitação de Alteração de Ocupação
                            $row['NRSOLICITACAO'] = \Geral\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 22, false);
                            $row['NRTIPOSOLICITACAO'] = SolicitacaoService::TP_SOLI_OCUPACAO;
                            $row['NRTIPOOCUPACAO'] = 1;
                            $row['NRTPMODALIDSAL'] = 1;
                            $row['DTINIOCUPACAO'] = $row['DTALTESALARIO'];
                            $erro = $this->solicitacaoService->executaInsertSolicitacao($row);
                        }
                    } else {
                        $erro = $this->solicitacaoService->executaInsertSolicitacao($row);
                    }

                    if(!$erro["Bloquear"]) $response->addNotification(new DTO\Response\Notification("1 row(s) persisted with success", DTO\Response\Notification::TYPE_SUCCESS));
                    else $response->addNotification(new DTO\Response\Notification($erro["Texto"], "ERROR"));
                }
                /* Edita uma solicitação */
                else {
                    parent::save($request, $response);
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function checaPerfilAcesso (DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();

        $parameters = array();
        foreach ($filters as $filter) {
            unset($filter['operator']);
            foreach ($filter as $key => $value) {
                if ($key == 'columnName') {
                    $parameters[$value] = $filter['value'];
                    unset($filter['columnName']);
                }
            }
        }
        
        if($parameters['NRTIPOSOLICITACAO'] != "") {
            $row['NRTIPOSOLICITACAO'] = $parameters['NRTIPOSOLICITACAO'];
            $row['NRVINCULOM'] = $parameters['NRVINCULOM'];
            $row['DTCOMPETENCIA'] = $parameters['DTCOMPETENCIA'];
            $erro = $this->solicitacaoService->validaPermissaoOperador($row);
        } else {
            $erro = array();
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erro', $erro));
    }    
    
    public function setPerfacestiposoli (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['NRNIVELAPROVASOLI'] = isset($row['NRNIVELAPROVASOLI'])?$row['NRNIVELAPROVASOLI']:null;
                $row['NRPERFACESAPROVPRI'] = isset($row['NRPERFACESAPROVPRI'])?$row['NRPERFACESAPROVPRI']:null;
                $row['NRPERFACESAPROVSEC'] = isset($row['NRPERFACESAPROVSEC'])?$row['NRPERFACESAPROVSEC']:null;
                if (array_key_exists('NRPERFACESTIPOSOLI', $row)) {
                    if ($row['NRPERFACESTIPOSOLI'] === '') {
                        $row['NRPERFACESTIPOSOLI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PERFACESTIPOSOLI', 1, 12, true);
                    }
                } else {
                    $row['NRPERFACESTIPOSOLI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PERFACESTIPOSOLI', 1, 12, true);
                }

                /* Prepara as informacoes que sao arrays */
                Util::rowArrayItemsToString($row, self::PERFACESTIPOSOLI_ARRAY_FIELDS);

                $this->dataSourceName = $this->dataSourceNamePerf;
                parent::save($request, $response);
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteSolicitacao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletePerfacestiposoli(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceNamePerf;
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setSolicitacaoHcmservices(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $rowAltered = $this->solicitacaoService->buildSolicitacaoRow($row);
                $this->solicitacaoService->setSolicitacaoHcmservices($rowAltered);
            }
            $response->addNotification(new DTO\Response\Notification(
                'Solicitação criada com sucesso', DTO\Response\Notification::TYPE_SUCCESS
            ));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function aprovaSolicitacaoHcmServices(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrsolicitacao = $row['NRSOLICITACAO'];
                $dsobservacao = $row['DSOBSAPROV'] ?? null;
                $cdoperadorAprovante = $this->hcmParameters->getCdoperador();
                $idAprovacao = $row['IDAPROVACAO'];

                if ($idAprovacao == self::IDAPROVACAO_APROVADO) {
                    $this->solicitacaoService->aprovaSolicitacaoHcmServices(
                        $nrsolicitacao, $cdoperadorAprovante, $dsobservacao
                    );
                } else {
                    $this->solicitacaoService->reprovaSolicitacaoHcmServices(
                        $nrsolicitacao, $cdoperadorAprovante, $dsobservacao
                    );
                }

                $response->addNotification(new DTO\Response\Notification('Solicitação processada com sucesso.', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function validaModelo(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $msg = null;
            $row = $request->getDataSet()->getRows()[0];
            $msg = $this->solicitacaoService->validaModelo($row);
            
            if(!empty($msg)){
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function aprovaSolicitacao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {

                $erro = $this->solicitacaoService->validaModeloAprovacao($row['NRTIPOSOLICITACAO']);
                
                if (!$erro['Bloquear']) {
                    /*if ($row['IDNIVELAPROV'] == 1) {
                        $row['IDAPROVACAOPRI'] = $row['IDAPROVACAO'] === "A" ? 'A' : 'R';
                        $row['DTAPROVACAOPRI'] = $this->hcmParameters->getDataAtual(true);
                        $row['CDOPERAPROVPRI'] = $this->hcmParameters->getCdoperador();
                        $row['DSOBSAPROVAPRI'] = isset($row['DSOBSAPROV'])?$row['DSOBSAPROV']:null;
                        $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                        $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                        $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                        parent::save($request, $response);
                    }

                    if ($row['IDNIVELAPROV'] == 2) {
                        $row['IDAPROVACAOSEC'] = $row['IDAPROVACAO'] === "A" ? 'A' : 'R';
                        $row['IDAPROVACAOPRI'] = ($row['IDAPROVACAOPRI'] == 'P')?$row['IDAPROVACAOSEC']:$row['IDAPROVACAOPRI'];
                        $row['DTAPROVACAOSEC'] = $this->hcmParameters->getDataAtual(true);
                        $row['CDOPERAPROVSEC'] = $this->hcmParameters->getCdoperador();
                        $row['DSOBSAPROVASEC'] = isset($row['DSOBSAPROV'])?$row['DSOBSAPROV']:null;
                        if($row['NRNIVELAPROVASOLI'] == 1 ||$row['NRNIVELAPROVASOLI'] == 3){
                            $row['DSOBSAPROVAPRI'] = isset($row['DSOBSAPROV'])?$row['DSOBSAPROV']:null;
                            $row['DTAPROVACAOPRI'] = $this->hcmParameters->getDataAtual(true);
                            $row['CDOPERAPROVPRI'] = $this->hcmParameters->getCdoperador();
                        }
                        $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                        $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                        $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                        if ($row['IDAPROVACAO'] === "A") {
                            $this->solicitacaoService->processarSolicitacaoAprovada($row);
                        } else {
                            $this->solicitacaoService->processarSolicitacaoReprovada($row);
                        }
                        parent::save($request, $response);
                    }*/

                    $idaprovacao = $this->solicitacaoService->aprovaFaseTramitacao($row['NRSOLICITACAO'], $row['NRTIPOSOLICITACAO'], $row['IDAPROVACAO'], isset($row['DSOBSAPROV']) ? $row['DSOBSAPROV'] : null);

                    if ($idaprovacao === "A") {
                        $this->solicitacaoService->processarSolicitacaoAprovada($row);
                    } else {
                        $this->solicitacaoService->processarSolicitacaoReprovada($row);
                    }

                    $row['IDAPROVACAOSEC'] = $idaprovacao;
                    $row['DTULTATU'] = $this->hcmParameters->getDataAtual(false);
                    $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();

                    parent::save($request, $response);
                } else {
                    $response->addMessage(new DTO\Response\Message($erro['Texto'], DTO\Response\Message::TYPE_ERROR));
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}