<?php
namespace HCM\Geral\Services;

use HCM\Geral\Repository\PreFechamentoIrrfRepository;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class PreFechamentoIrrfService {
    
    public function __construct(PreFechamentoIrrfRepository $preFechamentoIrrfRepository, HCMParameters $hcmParameters) {
        $this->preFechamentoIrrfRepository = $preFechamentoIrrfRepository;
        $this->hcmParameters = $hcmParameters;
        $this->retencaoTrabalhador = new RetencaoTrabalhador($preFechamentoIrrfRepository);
    }
    
    public function getRetencaoTrabalhador($parameters) {
        return $this->retencaoTrabalhador->getRetencaoTrabalhador($parameters);
    }
    
    public function getEventosApuradosVinculoIrrf($parameters) {
        return $this->retencaoTrabalhador->getEventosApuradosVinculoIrrf($parameters);
    }
}

class RetencaoTrabalhador {
    
    public function __construct(PreFechamentoIrrfRepository $preFechamentoIrrfRepository) {
        $this->preFechamentoIrrfRepository = $preFechamentoIrrfRepository;
    }
    
    public function getRetencaoTrabalhador($parameters) {
        $results = $this->preFechamentoIrrfRepository->getRetencaoTrabalhador($parameters);
        for($i=0; $i<count($results); $i++) {
            $results[$i]['CONCLUSAO'] = $this->getConclusao($results[$i]);
        }
        return $results;
    }
    
    private function getConclusao($params) {
        if( $params['IDSTATUS'] == null || $params['IDSTATUS'] == 'P' ){
            $conclusao = 'Não enviado';
        } else if( $params['IDSTATUS'] == 'E' || $params['IDSTATUS'] == 'R' ){
            $conclusao = 'Erro de processamento';
        } else if( $params['IDSTATUS'] == 'M' ){
            $conclusao = 'Em de processamento';
        } else if( $this->possuiDiferenca($params, 'BASE') ){
            $conclusao = 'Diferença na Base';
        } else if( $this->possuiDiferenca($params, 'IRRF') ){
            $conclusao = 'Diferença na retenção';
        } else if( $this->possuiDiferenca($params, 'ARREDONDAMENTO') ){
            $conclusao = 'Arredondamento';
        } else if( $this->possuiDiferenca($params, 'SEM_RETENCAO') ){
            $conclusao = 'Sem retenção de IRRF';
        } else {
            $conclusao = 'Sem diferença';
        }
        return $conclusao;
    }
    
    private function possuiDiferenca($params, $tipoDiferenca) {
        $base               = $params['VRBASE'];
        $baseEsocial        = $params['VRBASE_ESO'];
        $irrf               = $params['VRIRRF'];
        $irrfEsocial        = $params['VRIRRF_ESO'];
        $diferencaBase      = abs($params['DIFERENCA_BASE']);
        $diferencaIrrf      = abs($params['DIFERENCA_IRRF']);
        $status             = $params['IDSTATUS'];
   
        switch($tipoDiferenca) {
            case 'BASE':
                $result = ($base != $baseEsocial && $diferencaBase > '.01') &&
                          ($status == 'C' || $status == 'A');
                break;
            case 'IRRF':
                $result = ($irrf != $irrfEsocial && $diferencaIrrf > '.01') &&
                          ($status == 'C' || $status == 'A');
                break;
            case 'ARREDONDAMENTO':
                $result = ($base != $baseEsocial || $irrf != $irrfEsocial) &&
                          ($diferencaBase <= '.01' || $diferencaIrrf <= '.01') &&
                          ($status == 'C' || $status == 'A');
                break;
            case 'SEM_RETENCAO':
                $result = ($irrf == null && $diferencaIrrf == null || $irrf == 0 && $diferencaIrrf == 0) || 
                          ($irrf == 0 && $diferencaIrrf == null || $irrf == null && $diferencaIrrf == 0) &&
                          ($status == 'C' || $status == 'A');
                break;
            default:
                $result = false;
        }
        return $result;
    }
    
    public function getEventosApuradosVinculoIrrf($parameters) {
        $results = $this->preFechamentoIrrfRepository->getEventosApuradosVinculoIrrf($parameters);
        for($i=0; $i<count($results); $i++){
            if( $results[$i]['VALOR'] == $parameters['DIFERENCA_BASE'] && $parameters['CONCLUSAO'] == 'Diferença na Base' && ($results[$i]['VALOR'] - $parameters['DIFERENCA_BASE']) <= 0.1){
                $results[$i]['OBSERVACAO'] = 'S';
            }
        }
        return $results;
    }
}