<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\CnaeXcidService;

class CnaeXcidController extends Crud {

    protected $dataSourceName_Get = 'sst_cnaexcid_get';
    protected $dataSourceName_Set = 'sst_cnaexcid_set';
    protected $nmArquivoTemplateCon = 'cnaeXcid';

    protected $parameterBagColumns = array();

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, CnaeXcidService $cnaeXcidService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
        $this->cnaeXcidService = $cnaeXcidService;
    }

    public function getCNAExCID(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_Get;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setCNAExCID(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_Set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $existeCnaeXCid = $this->cnaeXcidService->validaCnaeXcid($row['CDCNAE'], $row['CDCID']);
                if(!$existeCnaeXCid){
                    if (array_key_exists('NRCNAECID', $row)) {
                        if ($row['NRCNAECID'] === '') {
                            $row['NRCNAECID'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_CNAE_CID', 1, 12, true);
                        }
                    } else {
                        $row['NRCNAECID'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_CNAE_CID', 1, 12, true);
                    }
                    parent::save($request, $response);
                }else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Relacionamento CNAE " . $row['CDCNAE'] . " e CID " . $row['CDCID'] . " já existentes!"  , \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                }
            }

            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaCnaeXCid(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $messages = $this->cnaeXcidService->importarPlanilhaCnaeXCid($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    
    public function baixarPlanilhaCnaeXcid(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            
            $file = $this->cnaeXcidService->baixarPlanilhaCnaeXcid(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->nmArquivoTemplateCon);
                
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteCNAExCID(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_Set;
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 