<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\SstFichamedica;
use Folha\Entities\SstExamefichamed;
use Folha\Entities\SstItemexamefichamed;
use Folha\Entities\SstDoencafichamedica;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Services\ESocialService;
use Zeedhi\Framework\DataSource\DataSet;

class FichaMedicaService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ESocialService $eSocialService) {
        $this->entityManager    = $entityManager;
        $this->hcmParameters    = $hcmParameters;
        $this->SstFichamedica   = Repositories::SST_FICHAMEDICA;
        $this->SstExamefichamed = Repositories::SST_EXAMEFICHAMED;
        $this->gpepessoa        = Repositories::GPE_PESSOA;
        $this->gpepessoah       = Repositories::GPE_PESSOAH;  
        $this->parcnegocio      = Repositories::PARCNEGOCIO;
        $this->eSocialService   = $eSocialService;
    }
    
    public function deleteFichaMedica($dataset) {
        
        $nrorg = $dataset["NRORG"];
        $nrfichamedica = $dataset["NRFICHAMEDICA"];
        $exames = SstExamefichamed::findByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
        $doenca = SstDoencafichamedica::findByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
        if (isset($exames[0])) {
            throw new \Exception(Messages::HCM05000_SST_FICHAMEDICA_EXISTS_EXAME_ERROR, 10);
        }
        if (isset($doenca[0])) {
            throw new \Exception(Messages::HCM05000_SST_FICHAMEDICA_EXISTS_DOENCA_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $fichamedica = SstFichamedica::findOneByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
            $this->entityManager->remove($fichamedica);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function deleteExameFichaMed($dataset) {
        $nrorg = $dataset["NRORG"];
        $nrexamefichamed= $dataset["NREXAMEFICHAMED"];
        $itemexame = SstItemexamefichamed::findByNrexamefichamedAndNrorg($nrexamefichamed, $nrorg);
        if (isset($itemexame[0])) {
            throw new \Exception(Messages::HCM05000_SST_EXAMEFICHAMED_EXISTS_ITEMEXAME_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $examefichamed = SstExamefichamed::findOneByNrexamefichamedAndNrorg($nrexamefichamed, $nrorg);
            $this->entityManager->remove($examefichamed);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function validaParametros($dataset, $response){
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrpacnegocio   = $this->VerificaNmMedico($dataset,$dtmescompetenc, $nrorg);
        $nrestrutgeren  = $dataset['NRESTRUTGEREN_LIST'];
        $nrestrutlegal  = $dataset['NRESTRUTLEGAL_LIST'];
        $nrVinculom     = $dataset['NRVINCULOM_LIST'];
        $arrayParametro =  $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosLegalGeren($nrorg, $nrestrutgeren, $nrestrutlegal, $dtmescompetenc, $nrVinculom);
        
        if($arrayParametro != null){
            try{    
                $retorno = array();
                $responseClone = clone $response;
                foreach($arrayParametro as $registro){                 
                    $nrVinculom = $registro["NRVINCULOM"];
                    $nrfichamedica = $this->setCadastroAutomaticoFichaMedica($dataset, $nrorg, $nrVinculom, $nrpacnegocio);
                              
                    if(array_key_exists('IDENVIAESOCIAL', $dataset) && $dataset['IDENVIAESOCIAL'] == 'S'){
                        $dataset["NRVINCULOM"] = $registro["NRVINCULOM"];
                        $dataset["NRFICHAMEDICA"] = $nrfichamedica;
                        $this->eSocialService->envioEsocialDescentralizado($dataset, $responseClone);
                     }
                     
                }

                foreach($responseClone->getDataSets() as $dataSet){
                    foreach($dataSet->getRows() as &$dataSet) { 
                        $retorno[] = $dataSet;
                    }
                    
                } 

                if(!empty($retorno)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retorno));
                }
            
            } catch (\Exception $e) {
                throw $e;
            }
            
        }

    }
    
    public function VerificaNmMedico($dataset,$dtmescompetenc, $nrorg){
        $nrinscrconsreg = $dataset['NRINSCRCONSREG'];
        if($nrinscrconsreg != null){
            $objNrInscricao = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaPessoaNrinscrconsreg($nrorg, $nrinscrconsreg, $dtmescompetenc);
            if($objNrInscricao == null){
                $nrpacnegocio =  $this->salvaParcNegocio($dataset,$nrorg);
                return $nrpacnegocio;
            }else{
                $nrpessoa = $objNrInscricao[0]["nrpessoa"];
                $objNrpacrnegocio = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrorg' =>$nrorg, 'nrpessoa'=>$nrpessoa));
                $nrpacnegocio = $objNrpacrnegocio->getNrparcnegocio();
                return $nrpacnegocio;
            }
        }
    }
    
    public function setCadastroAutomaticoFichaMedica($dataset,$nrorg ,$nrvinculom, $nrpacnegocio){
        try {
            $this->entityManager->beginTransaction();
            
            $dtconclusao  = DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true);
            $dtPrevista   = (isset($dataset['DTHRPREVFICHAMED'])) ? DateUtil::getDataDeString($dataset['DTHRPREVFICHAMED'] , DateUtil::FORMATO_BRASILEIRO, true) : $dtconclusao;
            
            $envioFichaMedica = new $this->SstFichamedica;
            $envioFichaMedica->setNrorg($nrorg);
            $envioFichaMedica->setNrvinculom($nrvinculom);
            $envioFichaMedica->setNrinscrconselhoatend($dataset['NRINSCRCONSREG']);
            $envioFichaMedica->setNrfichamedica(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_FICHAMEDICA', 1, 12, true));
            $envioFichaMedica->setNrnaturezaexame($dataset['NRNATUREZAEXAME']);
            $envioFichaMedica->setDtconclfichamed($dtconclusao);
            $envioFichaMedica->setDthrprevfichamed($dtPrevista);
            $envioFichaMedica->setIdativo('S');
            $envioFichaMedica->setNrparcnegocio($nrpacnegocio);
            
            // set auditoria
            $envioFichaMedica->setDtinclusao(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorginclusao($nrorg);
            $envioFichaMedica->setDtultatu(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorgultatu($nrorg);
            $this->entityManager->persist($envioFichaMedica);
            $this->entityManager->flush($envioFichaMedica);
            $nrfichamedica = $envioFichaMedica->getNrfichamedica();
            
            $this->setCadastroAutomaticoExame($dataset,$nrorg,$nrfichamedica,$nrpacnegocio);
            
            $this->entityManager->commit();
            
            return $nrfichamedica;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
     public function setCadastroAutomaticoExame($dataset,$nrorg,$nrfichamedica,$nrpacnegocio){
        try {
            if (isset($dataset['NRPERFILEXAME']) && $dataset['NRPERFILEXAME'] != '') {
                /*Cadastra exames da ficha a partir do perfil de exames se informado*/
                $itens = $this->entityManager->getRepository(Repositories::SST_ITEMPERFILEXAME)->findBy(array('nrorg' =>$nrorg, 'nrperfilexame'=>$dataset['NRPERFILEXAME']));
                
                foreach ($itens as $item) {
                    $exame = $this->entityManager->getRepository(Repositories::SST_EXAME)->findOneBy(array('nrorg' =>$nrorg, 'nrexame'=>$item->getNrexame()));
                    
                    $envioCadExame = new $this->SstExamefichamed;
                    $envioCadExame->setNrorg($nrorg);
                    $envioCadExame->setCdproctuss($exame->getCdproctuss());
                    $envioCadExame->setNrexame($exame->getNrexame());
                    $envioCadExame->setNmexamefichamed($exame->getNmexame());
                    $envioCadExame->setNrexamefichamed(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EXAMEFICHAMED', 1, 12, true));
                    $envioCadExame->setNrfichamedica($nrfichamedica);
                    $envioCadExame->setIdativo('S');
                    $envioCadExame->setNrparcnegocio($nrpacnegocio); 
                    $envioCadExame->setDtrealizacaoexame(DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true));
                    
                    // set auditoria
                    $envioCadExame->setDtinclusao(DateUtil::getDataAtual());
                    $envioCadExame->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $envioCadExame->setNrorginclusao($nrorg);
                    $envioCadExame->setDtultatu(DateUtil::getDataAtual());
                    $envioCadExame->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envioCadExame->setNrorgultatu($nrorg);
                    $this->entityManager->persist($envioCadExame);
                    $this->entityManager->flush($envioCadExame);
                    
                    unset($envioCadExame);
                }
            } else {
                /*Cadastra exames da fihca a partir dos exames informandos no cadastro automatico*/
                $envioCadExame = new $this->SstExamefichamed;
                $envioCadExame->setNrorg($nrorg);
                $envioCadExame->setCdproctuss($dataset['CDPROCTUSS']);
                $envioCadExame->setNrexame($dataset['NREXAME']);
                $envioCadExame->setNmexamefichamed($dataset['NMEXAMEFICHAMED']);
                $envioCadExame->setNrexamefichamed(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EXAMEFICHAMED', 1, 12, true));
                $envioCadExame->setNrfichamedica($nrfichamedica);
                $envioCadExame->setIdativo('S');
                $envioCadExame->setNrparcnegocio($nrpacnegocio); 
                $envioCadExame->setDtrealizacaoexame((DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true)));
                
                // set auditoria
                $envioCadExame->setDtinclusao(DateUtil::getDataAtual());
                $envioCadExame->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $envioCadExame->setNrorginclusao($nrorg);
                $envioCadExame->setDtultatu(DateUtil::getDataAtual());
                $envioCadExame->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envioCadExame->setNrorgultatu($nrorg);
                $this->entityManager->persist($envioCadExame);
                $this->entityManager->flush($envioCadExame);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function salvaParcNegocio($dataset,$nrorg) {
        try{
            $obj = new $this->parcnegocio;
            $obj->setIdinstituicao('N');
            $obj->setIdpessoafisica('S');
            $obj->setIdativo('S');
            $obj->setNrorg($nrorg);
            $obj->setNrparcnegocio(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'PARCNEGOCIO', 1, 12, true));
            $obj->setNmprincipalparc($dataset['NMPESSOA']);
            $obj->setNmsecundariparc($dataset['NMPESSOA']);
            
            // set auditoria
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorginclusao($nrorg);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($nrorg);
            $this->entityManager->persist($obj);
            $this->entityManager->flush($obj);
            $nrpacnegocio = $obj->getNrparcnegocio();
            $this->inserePessoaMestre($dataset,$nrorg,$nrpacnegocio);
            return $nrpacnegocio;
            
    } catch(\Exception $e) {
        throw $e;
        }
    }
    
    public function inserePessoaMestre($dataset,$nrorg,$nrpacnegocio){
        
        try{
        
            $pessoamestre = new $this->gpepessoa;
            $pessoamestre->setNrorg($nrorg);
            $pessoamestre->setNrparcnegocio($nrpacnegocio);
            $pessoamestre->setNrpessoa(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PESSOA', 1, 12, true));
            
             // set auditoria
            $pessoamestre->setDtinclusao(DateUtil::getDataAtual());
            $pessoamestre->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorginclusao($nrorg);
            $pessoamestre->setDtultatu(DateUtil::getDataAtual());
            $pessoamestre->setCdoperultatu($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorgultatu($nrorg);
            $this->entityManager->persist($pessoamestre);
            $this->entityManager->flush($pessoamestre);
            $nrPessoa = $pessoamestre->getNrpessoa();
            $this->inserePessoaHistorico($dataset,$nrorg,$nrPessoa);
        
    } catch(\Exception $e) {
        throw $e;
        }
    }
    
    public function inserePessoaHistorico($dataset,$nrorg,$nrPessoa){

        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        
        try{
        
            $pessoamestre = new $this->gpepessoah;
            $pessoamestre->setNrorg($nrorg);
            $pessoamestre->setNmPessoa($dataset['NMPESSOA']);
            $pessoamestre->setNrpessoa($nrPessoa);
            $pessoamestre->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtexconsreg(DateUtil::getDataDeString($dataset['DTEXCONSREG'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtinivigencia(DateUtil::getDataDeString($dataset['DTINIVIGENCIA'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtfimvigencia(DateUtil::getDataDeString($dataset['DTFIMVIGENCIA'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setNrinscrconsreg($dataset['NRINSCRCONSREG']);
            $pessoamestre->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PESSOAH', 1, 12, true));
            $pessoamestre->setSgufconsreg($dataset['SGUFCONSREG']);
            $pessoamestre->setNrcpfpessoa($dataset['NRCPFPESSOA']); 
            $pessoamestre->setNrconselhoreg($dataset['NRCONSELHOREG']); 
            $pessoamestre->setSgufconsreg($dataset['SGUFCONSREG']);
            
             // set auditoria
            $pessoamestre->setDtinclusao(DateUtil::getDataAtual());
            $pessoamestre->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorginclusao($nrorg);
            $pessoamestre->setDtultatu(DateUtil::getDataAtual());
            $pessoamestre->setCdoperultatu($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorgultatu($nrorg);
            $this->entityManager->persist($pessoamestre);
            $nrPessoa = $pessoamestre->getNrpessoa();
            $this->entityManager->flush($pessoamestre);
        
    } catch(\Exception $e) {
        throw $e;
        }
    }
    
    
}