<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 28/03/2016
 * Time: 11:29
 */

namespace HCM\Util;

use \Zeedhi\Framework\DTO\Response;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;

class HCM {

    protected $operadorService;
    protected $hcmParameters;
    protected $parametros;
    private static $instancia;

    public function __construct(\HCM\Geral\Services\OperadorService $operadorService, HCMParameters $hcmParameters) {
        $this->operadorService = $operadorService;
        $this->hcmParameters   = $hcmParameters;

        self::$instancia = $this;
    }

    public static function getInstancia() {
        return self::$instancia;
    }

    public function factory($request, $response) {

        try {

            $filter = $request->getFilterCriteria()->getConditions();
            $cdoperador = $filter[0]['value'];
            $this->parametros = $this->operadorService->factoryOperador($cdoperador);
            $this->setParameters($this->parametros);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('hcmFactory', $this->parametros));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getOperadores($request, $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $nrorg = $filter[0]['value'];

            $operadores = $this->operadorService->carregaOperadoresOrg($nrorg);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getOperadores', $operadores));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    private function setParameters($parameters) {
        foreach ($parameters as $key => $value) {
            $this->hcmParameters->set($key, $value);
        }
    }

    public function getFilterPlaceHolder($filters) {
        $conditions = array();
        foreach($filters as $value) {
            $conditions[] = 'UPPER(' . $value["columnName"] . ') ' . $value["operator"] . " UPPER('" . $value["value"] . "')";
        }

        if (empty($conditions)) {
            $conditions[] = "1 = 1";
        }
        
        return implode(" AND ", $conditions);
    }

    public static function loadReport($reportUrl) {
        $cURLRequest = new cURLRequest($reportUrl);
        $report = $cURLRequest->request('');

        $requestInfo = $cURLRequest->getRequestInfo();    
        if ($requestInfo['http_code'] !== HTTPInterface::STATUS_CODE_OK) {
            throw new \Exception(Messages::HCM01000_FPA_ERROR_SAVE_REPORT, 1);
        }  

        return $report;
    }

    public static function gravarArquivo($fileName, $fileContent, $writing = false) {
        try {
            $filePath = self::getInstancia()->hcmParameters->getTempDirectory().DIRECTORY_SEPARATOR.$fileName;
            if (!$writing)
                file_put_contents($filePath, $fileContent);
            else
                file_put_contents($filePath, $fileContent, FILE_APPEND);

            return $filePath;
        } catch(\Exception $e) {
            throw new \Exception(Messages::HCM00000_GER_NAO_POSSIVEL_GRAVAR_ARQUIVO, 1);            
        }
    }

    public function getBaseUrlFileServer($request, $response) {
        $url = [$this->hcmParameters->getFileServerDirectory()];

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('baseUrlFileServer', $url));
    }

}