<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Logic\ESocial\GerenciaEnvios;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1000 - Informações do Empregador/Contribuinte.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1000
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1000, Informações do
     * Empregador/Contribuinte.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente)
    {
        $S1000 = GpeEsocialRegistro::dataSetEsocial_S1000($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                                          $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente);

        foreach ($S1000 as &$row) {
            /*Verifica pelo log de envios (GPE_ESOCIAL_ENVIO) se o evento/chave já foi enviado para definir se é um envio de inclusão ou alteração*/
            $operacao = new GerenciaEnvios('S-1000', $versaoEsocial, $row['CHAVE'], null, $nrorg);
            $operacao = $operacao->validaTipoEnvio();
            $row['INCLUSAO'] = $operacao['INCLUSAO'];
            $row['ALTERACAO'] = $operacao['ALTERACAO'];

            /*Aqui são geradas as Software Houses para cada Empregador*/
            $softwareHouses = GpeEsocialRegistro::s1000_softwareHouse();
            $contSH = 1;
            foreach ($softwareHouses as &$softwareHouse) {
                if ($versaoEsocial === \ESocial::VERSAO_2_0) {
                    $row["SOFTWAREHOUSE"][$contSH] =
                        $row['INCLUSAO'] == 1 ?
                            array("_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_CNPJSOFTHOUSE" => $softwareHouse["CNPJSOFTWAREHOUSE"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_NMRAZAO" => $softwareHouse["SOFTNOMERAZAO"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_NMCONT" => $softwareHouse["SOFTNOMECONTATO"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_TELEFONE" => $softwareHouse["TELEFONE"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_EMAIL" => $softwareHouse["SOFTEMAIL"])
                        :
                            array("_EVTINFOEMPREGADOR_INFOEMPREGADOR_ALTERACAO_INFOCADASTRO_SOFTWAREHOUSE_CNPJSOFTHOUSE" => $softwareHouse["CNPJSOFTWAREHOUSE"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_ALTERACAO_INFOCADASTRO_SOFTWAREHOUSE_NMRAZAO" => $softwareHouse["SOFTNOMERAZAO"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_ALTERACAO_INFOCADASTRO_SOFTWAREHOUSE_NMCONT" => $softwareHouse["SOFTNOMECONTATO"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_ALTERACAO_INFOCADASTRO_SOFTWAREHOUSE_TELEFONE" => $softwareHouse["TELEFONE"],
                                "_EVTINFOEMPREGADOR_INFOEMPREGADOR_ALTERACAO_INFOCADASTRO_SOFTWAREHOUSE_EMAIL" => $softwareHouse["SOFTEMAIL"]);
                }
                else {
                    $row["SOFTWAREHOUSE"][$contSH] =
                        array("_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_CNPJSOFTWAREHOUSE" => $softwareHouse["CNPJSOFTWAREHOUSE"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_NOMERAZAO" => $softwareHouse["SOFTNOMERAZAO"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_NOMECONTATO" => $softwareHouse["SOFTNOMECONTATO"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_TELEFONE" => $softwareHouse["TELEFONE"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_CODMUNICIPIO" => $softwareHouse["CODMUNICIPIO"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_UF" => $softwareHouse["UF"],
                            "_EVTINFOEMPREGADOR_INFOEMPREGADOR_INCLUSAO_INFOCADASTRO_SOFTWAREHOUSE_EMAIL" => $softwareHouse["SOFTEMAIL"]);
                }
                $contSH++;
            }
        }

        return $S1000;
    }

}