<?php
namespace Folha\Logic\GravarPonto;

/**
 * Regras de negócio relacionadas ao filtro da tela de AJ (Apuração de Jornada).
 *
 * @author TulioMagalhes
 */
class FiltroAJ {

    public static function getDadosOperadorFiltroAJ() {
        $ambiente = \Geral\Util\Ambiente::getInstancia();
        
        /**
         * Código do funcionário relacionado ao operador logado 
         */
        $cdempresa = $ambiente->getOperador()->getFunc()->getCdempresa();
        $cdfunc = $ambiente->getOperador()->getFunc()->getCdfunc();
        
        /**
         * Verifica permissão de alterar o ponto 
         * de outros funcionários de acordo com os grupos de operador
         * aos quais o operador está relacionado.
         */
        $cdoperador = $ambiente->getCdOperador();
        $nrorg = $ambiente->getNrorg();

        $gruposDoOperador = \Geral\Entities\Opergrupop::findByCdoperadorAndNrorg($cdoperador, $nrorg);

        $permitePesqFunc = false;
        foreach ($gruposDoOperador as $grupo) {
            $cdgrupoper = $grupo->getCdgrupoper();

            $autgrupop = \Producao\Entities\Autgrupop::findByCdgrupoperAndIdautgrupopAndNrorg($cdgrupoper, 'DPAP', $nrorg);
            if (count($autgrupop) > 0) {
                $permitePesqFunc = true;
            }
        }

        /** 
         * Código do tipo de admissão do funcionário relacionado ao operador 
         * logado.
         */
        $cdtpadmissao = $ambiente->getOperador()->getFunc()->getCdtpadmissao();
        
        /**
         * Verifica o período atual de acordo com a data de hoje
         */
        $dtperiodo = self::getPeriodoapurapontoAtual($nrorg, $cdempresa, $cdfunc, $cdtpadmissao);
        
        /* Monta o array contendo os valores que serão retornados 
         */
        $dadosfiltrocda = array(
            'cdfunc'          => $cdfunc,
            'permitepesqfunc' => $permitePesqFunc,
            'cdtpadmissao'    => $cdtpadmissao,
            'dtperiodo'       => $dtperiodo
        );
        
        return $dadosfiltrocda;

    }
    
    public static function getPeriodoapurapontoAtual($nrorg, $cdempresa, $cdfunc, $cdtpadmissao) {
        $func = \Geral\Entities\Funcionario::findOneByCdempresaAndCdfuncAndNrorg($cdempresa, $cdfunc, $nrorg);
        if(is_object($func)) {
            $dtdeslfunc = $func->getDtDeslFunc();
        } else {
            $dtdeslfunc = '';
        } 
        
        if(is_null($cdtpadmissao) || $cdtpadmissao == '') {
            $cdtpadmissao = "C";
        }

        $fpa_periodoapuraponto = \Folha\Repositories\Fpaperiodoapuraponto::consultaPeriodoAtual($cdtpadmissao,  $nrorg);
    
        $idperiodoapuraponto = $fpa_periodoapuraponto[0]['IDPERIODOAPURAPONTO'];
        $dtperiodo  = $fpa_periodoapuraponto[0]['DTINICIALAPURAPONTO'];
        $dtperiodo .= ',';
        $dtperiodo .= $fpa_periodoapuraponto[0]['DTFINALAPURAPONTO'];
        $dtperiodo .= ',';
        $dtperiodo .= $fpa_periodoapuraponto[0]['IDPERIODOAPURAPONTO'];
        
        return $dtperiodo;
    }
    
    public static function getTipoAdmissaoNomeFunc($nrorg, $cdempresa, $cdfunc) {
        $func = \Geral\Entities\Funcionario::findOneByCdempresaAndCdfuncAndNrorg($cdempresa, $cdfunc, $nrorg);

        $cdtpadmissao = '';
        $nmfunc = '';
        
        if(is_object($func)) {
            $cdtpadmissao = $func->getCdtpadmissao();
            $nmfunc = $func->getNmfunc();
        }

        $tipoAdmissaoNomeFunc = array(
            'nmfunc'       => $nmfunc,
            'cdtpadmissao' => $cdtpadmissao
        );

        return $tipoAdmissaoNomeFunc;
    }

    public static function getPermicaoHoraDeAlmoco($cdtpadmissao)
    {
        if(($cdtpadmissao =='S') or ($cdtpadmissao =='E') or ($cdtpadmissao =='P'))
        {
            return false;
        }else {
            return true;
        }
    }
    
    public static function returnBoolExistsPonto($cdEmpresa, $cdFunc, $dtBatidaPto, $nrOrg, $nrPonto)
    {
        $where = "CDEMPRESA = ?
              AND CDFUNC = ?
              AND TO_CHAR(DTBATIDAPTO, 'DD/MM/YYYY') = ?
              AND NRORG = ?";
                
        $params = array($cdEmpresa, $cdFunc, $dtBatidaPto, $nrOrg);
        
        $batidaPonto = \Folha\Entities\Batidaponto::getObjectsByWhere($where, $params);
        $fpaPonto    = \Folha\Entities\FpaPonto::findOneByNrpontoAndNrorg($nrPonto, $nrOrg);
        
        if(!empty($batidaPonto) || is_object($fpaPonto)) {
           return true;
        }
        return false;
    }
    
}

?>